﻿using System.IO;
using System.Security.Cryptography.Xml;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Text;
using System.Xml;
using System.Linq;

namespace IncomesRegisterSignatureSample
{
    public class DigitalSignatureMessageInspector : IClientMessageInspector
    {
        public void AfterReceiveReply(ref Message reply, object correlationState)
        {
            
        }

        public object BeforeSendRequest(ref Message request, IClientChannel channel)
        {            
            AddSignature(ref request);
            return null;
        }

        private void AddSignature(ref Message message)
        {            
            using (var memoryStream = new MemoryStream())
            {
                // Create a XmlDocument from the inspected message.
                var writer = XmlDictionaryWriter.CreateTextWriter(memoryStream);
                message.WriteMessage(writer);
                writer.Flush();
                var messageBody = Encoding.UTF8.GetString(memoryStream.ToArray());

                var xdoc = new XmlDocument {PreserveWhitespace = true};                            
                xdoc.LoadXml(messageBody);

                // Extract soap body content from the document and sign the payload.
                var soapBody = xdoc.SelectNodes("//*[local-name() = 'Body']")[0];
                var soapBodyContent = soapBody.FirstChild;

                var signedDocument = new XmlDocument { PreserveWhitespace = true };
                signedDocument.LoadXml(soapBodyContent.OuterXml);
                // This example expects the certificate is stored in certificatestore.
                Signature.SignXmlDocument(signedDocument, "certificate_friendlyname_here");
                
                // Replace original payload with signed payload.
                soapBody.RemoveChild(soapBodyContent);
                var importNode = soapBody.OwnerDocument.ImportNode(signedDocument.FirstChild, true);
                soapBody.AppendChild(importNode);

                // Now that the message was read, it needs to be recreated.                
                // As the message body was modified, need to re-encode it
                messageBody = xdoc.OuterXml;
                var newMessageStream = new MemoryStream(Encoding.UTF8.GetBytes(messageBody));
                var reader = XmlDictionaryReader.CreateTextReader(newMessageStream, XmlDictionaryReaderQuotas.Max);
                var newMessage = Message.CreateMessage(reader, int.MaxValue, message.Version);
                newMessage.Properties.CopyProperties(message.Properties);
                // Replace original messageobject with new messageobject containing signed payload.
                message.Close();
                message = newMessage;                
            }
        }
    }
}
