﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Threading.Tasks;

namespace IncomesRegisterSignatureSample
{
    /// <summary>
    /// Provides a set of supported XML signature configurations.
    /// </summary>
    public class SignatureConfiguration
    {
        private SignatureConfiguration(
            string referenceUri,
            string signingAlgorithmName,
            string digestAlgorithmName,
            string canonicalizationMethod)
        {
            ReferenceUri = referenceUri;
            SigningAlgorithmName = signingAlgorithmName;
            DigestAlgorithmName = digestAlgorithmName;
            CanonicalizationMethod = canonicalizationMethod;                        

        }
        public string ReferenceUri { get; set; }
        public string SigningAlgorithmName { get; }
        public string DigestAlgorithmName { get; }
        public string CanonicalizationMethod { get; }                
        
        /// <summary>
        /// Signature configuration providing digest with SHA256, signature with RSA-SHA256, canonicalizes with Excluding C14N with no comments.
        /// </summary>
        /// <remarks>
        /// Constant values for XmlDsigSHA256Url and XmlDsigRSASHA256Url are supported from framework version 4.6.2. 
        /// </remarks>
        public static SignatureConfiguration SHA256_RSASHA256_ExclusiveC14N_NoComments =
            new SignatureConfiguration(
                string.Empty,
                SignedXml.XmlDsigRSASHA256Url,
                SignedXml.XmlDsigSHA256Url,
                SignedXml.XmlDsigExcC14NTransformUrl);
    }
}
